 /*********************************************************************/
 /* programme: series.c                                               */
 /*                                                                   */
 /* utilit: Programme qui gnre les series utilisees dans           */
 /*          Hong & White 2005, Econometrica                          */
 /*                                                                   */
 /*                                                                   */
 /* fonction utilise: kiss.c ( gnration de nombres distribus      */  
 /*                                  uniformment sur [0,1]     )     */
 /*  Lois:   (with Gaussian innovations                               */
/*        0-  White noise                                             */
 /*       1-  AR(1)                                                   */
 /*       2-  ARCH(1)                                                 */                                         
 /*       3-  Threshold GARCH(1,1)                                    */                                              
 /*       4-  Bilinear AR(1)                                          */
 /*       5-  Nonlinear MA(1)                                         */ 
 /*       6-  Threshold AR(1)                                         */  
 /*       7-  Frational AR(1)                                         */  
 /*       8-  Sign AR(1)                                              */
 /*                                                                   */
 /*                                                                   */
 /* par: Kilani Ghoudi et Bruno Remillard, le 23 novembre        2005 */
 /*********************************************************************/






#include <stdio.h>
#include <stdlib.h>  
#include <math.h>  
#include <malloc.h> 
#include <sys/types.h>
#include <time.h>
#include "kiss.c"

#define pi 3.142592654


   /*  Gaussian variate */
   double rnorm() {
      double u1, u2, R, S;
      u1 = kiss(); 
      u2 = kiss();
      R = pow((double)-2*log(u1),(double)0.5);
      S = cos(2*pi*u2); 
      return R*S;}

     /*********************************************************************/
     /*********************************************************************/

   void laws(double *x, int n, int loi, double theta)
   {
      double sigma, temp;
      double u1, u2 ,u3;
      int k,i;
   
     /*********************************************************************/
   
      if (loi == 0)  /** Gaussian white noise **/
      {
         for(k=0;k<n;k++){ x[k] = rnorm();}
      }
   
     /*********************************************************************/ 
   
      if (loi == 1)  /** AR(1) with Gaussian errors **/
      {
         temp = 0.0;
         for(i=0;i<100;i++)
            temp = 0.3*temp + rnorm();
      
         x[0] = temp;
         for(k=1;k<n;k++)
            x[k] = 0.3*x[k-1] + rnorm();
      }
      
      /*********************************************************************/ 
      
      else if (loi == 2)  /** ARCH(1) with Gaussian errors **/
      {
         temp = 0.0;
         for(i=0;i<100;i++)
         { temp = rnorm()*sqrt(1.0+ 0.8*temp*temp);}
      
         x[0] = temp;
         for(k=1;k<n;k++)
            x[k] = rnorm()*sqrt(1.0+ 0.8*x[k-1]*x[k-1]);
      
      }
      
      /*********************************************************************/ 
      
      else if (loi == 3) /** Threshold GARCH(1,1) with Gaussian errors**/
      {
         temp = 0.0;
         sigma = 0.0;
         u1 = rnorm();
      
         for(i=0;i<100;i++)
         {
            u2 = rnorm();
            sigma = sqrt(0.25 + 0.6*sigma*sigma+ 0.5*temp*temp*(u1 <0) + 0.2*temp*temp*(u1 >= 0));
            temp = u2*sigma ;
            u1 = u2;
         }
         x[0] = temp; 
      
         for(k=1;k<n;k++)
         {
            u2 = rnorm();
            sigma = sqrt(0.25 + 0.6*sigma*sigma+ x[k-1]*x[k-1]*(0.5*(u1 <0) + 0.2*(u1 >= 0)) );
            x[k] = u2*sigma;
            u1 = u2;
         }
      
      }
   
      /*********************************************************************/ 
   
      if (loi == 4)  /** Bilinear AR(1) with Gaussian errors **/
      {
         temp = 0.0;
         u1 = rnorm();
      
         for(i=0;i<100;i++)
         {
            u2 = rnorm();
            temp = 0.8*temp*u1 + u2;
            u1 = u2;
         }
      
         x[0] = temp;
         for(k=1;k<n;k++)
         {
            u2 = rnorm();
            x[k] = 0.8*x[k-1]*u1 + u2;
            u1 = u2;
         }
      }
      
      /*********************************************************************/
      
      else if (loi == 5)   /** Nonlinear MA(1) **/
      {
         u1 = rnorm();
      
      
         for(k=0;k<n;k++)
         { 
            u2 = rnorm();
            x[k] = 0.8*u1*u1+ u2;
            u1 = u2;
         }
      
      
      }
      
      /*********************************************************************/ 
      
      
      else if (loi == 6)  /** Threshold AR(1) **/
      {
         u1 = rnorm();
         temp=0.0;
         for(k=0;k<100;k++)
         { 
            u2 = rnorm();
            u3=temp;
            temp = u2 -(u3 <= 1)*0.5*u3+(u3>1)*0.4*u3;
         }
         x[0]=temp;
         for(k=1;k<n;k++)
         { 
            u2 = rnorm();
            u3=x[k-1];
            x[k] = u2- (u3 <= 1)*0.5*u3+(u3>1)*0.4*u3;
         
         }
      
      }
      
      /*********************************************************************/      
      

      else if (loi == 7)       /** Fractional AR(1) **/
      {
         temp=0.0;
         for(k=0;k<100;k++)
         { 
            u2 = rnorm();
            temp = 0.8*sqrt(fabs(temp))+u2;
            
         }
         x[0]=temp;
         for(k=1;k<n;k++)
         { 
            u2 = rnorm();
            x[k] = 0.8*sqrt(fabs(x[k-1]))+u2;
         }
      
      }
      
      /*********************************************************************/
      
      else if (loi == 8)  /** Modele SIGN AR(1) **/
      {
         u1 = rnorm();
         temp= 0.0;
         for(k=0;k<100;k++)
         { 
            u3=temp;
			u2=rnorm();
            temp= (2.0*(u3>0)-1) + 0.43*u2;
         }
         x[0]=temp;
         for(k=1;k<n;k++)
         { 
            u2=rnorm();
            x[k] =(2.0*(x[k-1]>0)-1) + 0.43*u2;      
         }
      
      }
   
   
   }